// LICENSE : MIT
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TextLintTester = void 0;
const assert = __importStar(require("assert"));
const test_util_1 = require("./test-util");
const textlint_1 = require("textlint");
const feature_flag_1 = require("@textlint/feature-flag");
/* eslint-disable no-invalid-this */
const globalObject = global;
const describe = typeof globalObject.describe === "function"
    ? globalObject.describe
    : function (_text, method) {
        return method.apply(this);
    };
const it = typeof globalObject.it === "function"
    ? globalObject.it
    : function (_text, method) {
        return method.apply(this);
    };
/* eslint-enable no-invalid-this */
/**
 * get fixer function from ruleCreator
 * if not found, throw error
 * @param {Function|Object} ruleCreator
 * @param {string} ruleName
 */
function assertHasFixer(ruleCreator, ruleName) {
    if (typeof ruleCreator.fixer === "function") {
        return;
    }
    if (typeof ruleCreator === "function") {
        return;
    }
    throw new Error(`Not found \`fixer\` function in the ruleCreator: ${ruleName}`);
}
function assertTestConfig(testConfig) {
    assert.notEqual(testConfig, null, "TestConfig is null");
    assert.notEqual(Object.keys(testConfig).length === 0 && testConfig.constructor === Object, true, "TestConfig is empty");
    assert.ok(Array.isArray(testConfig.rules), "TestConfig.rules should be an array");
    assert.ok(testConfig.rules.length > 0, "TestConfig.rules should have at least one rule");
    testConfig.rules.forEach((rule) => {
        assert.ok(rule.hasOwnProperty("ruleId"), "ruleId property not found");
        assert.ok(rule.hasOwnProperty("rule"), "rule property not found");
    });
    if (typeof testConfig.plugins !== "undefined") {
        assert.ok(Array.isArray(testConfig.plugins), "TestConfig.plugins should be an array");
        testConfig.plugins.forEach((plugin) => {
            assert.ok(plugin.hasOwnProperty("pluginId"), "pluginId property not found");
            assert.ok(plugin.hasOwnProperty("plugin"), "plugin property not found");
        });
    }
}
function isTestConfig(arg) {
    if (arg.hasOwnProperty("rules")) {
        return true;
    }
    if (typeof arg.fixer === "function" || typeof arg === "function") {
        return false;
    }
    return true;
}
function createTestRuleSet(testConfigRules) {
    const testRuleSet = {
        rules: {},
        rulesOptions: {}
    };
    testConfigRules.forEach((rule) => {
        const ruleName = rule.ruleId;
        const ruleOptions = rule.options;
        testRuleSet.rules[ruleName] = rule.rule;
        testRuleSet.rulesOptions[ruleName] = ruleOptions;
    });
    return testRuleSet;
}
function createTestPluginSet(testConfigPlugins) {
    const testPluginSet = {
        plugins: {},
        pluginOptions: {}
    };
    testConfigPlugins.forEach((plugin) => {
        const pluginName = plugin.pluginId;
        const pluginOptions = plugin.options;
        testPluginSet.plugins[pluginName] = plugin.plugin;
        testPluginSet.pluginOptions[pluginName] = pluginOptions;
    });
    return testPluginSet;
}
class TextLintTester {
    constructor() {
        if (typeof feature_flag_1.coreFlags === "object") {
            feature_flag_1.coreFlags.runningTester = true;
        }
    }
    testValidPattern(name, param, valid) {
        const text = typeof valid === "object" ? valid.text : valid;
        const inputPath = typeof valid === "object" ? valid.inputPath : undefined;
        const ext = typeof valid === "object" && valid.ext !== undefined ? valid.ext : ".md";
        const textlint = new textlint_1.TextLintCore();
        if (isTestConfig(param)) {
            const testRuleSet = createTestRuleSet(param.rules);
            textlint.setupRules(testRuleSet.rules, testRuleSet.rulesOptions);
            if (param.plugins !== undefined) {
                const testPluginSet = createTestPluginSet(param.plugins);
                textlint.setupPlugins(testPluginSet.plugins, testPluginSet.pluginOptions);
            }
        }
        else {
            const options = typeof valid === "object"
                ? valid.options
                : // just enable
                    true;
            textlint.setupRules({
                [name]: param
            }, {
                [name]: options
            });
        }
        it(inputPath || text, () => {
            return test_util_1.testValid({ textlint, inputPath, text, ext });
        });
    }
    testInvalidPattern(name, param, invalid) {
        const errors = invalid.errors;
        const inputPath = invalid.inputPath;
        const text = invalid.text;
        const ext = invalid.ext !== undefined ? invalid.ext : ".md";
        const textlint = new textlint_1.TextLintCore();
        if (isTestConfig(param)) {
            const testRuleSet = createTestRuleSet(param.rules);
            textlint.setupRules(testRuleSet.rules, testRuleSet.rulesOptions);
            if (Array.isArray(param.plugins)) {
                const testPluginSet = createTestPluginSet(param.plugins);
                textlint.setupPlugins(testPluginSet.plugins, testPluginSet.pluginOptions);
            }
        }
        else {
            const options = invalid.options;
            textlint.setupRules({
                [name]: param
            }, {
                [name]: options
            });
        }
        it(inputPath || text, () => {
            return test_util_1.testInvalid({ textlint, inputPath, text, ext, errors });
        });
        // --fix
        if (invalid.hasOwnProperty("output")) {
            it(`Fixer: ${inputPath || text}`, () => {
                if (isTestConfig(param)) {
                    param.rules.forEach((rule) => {
                        assertHasFixer(rule.rule, rule.ruleId);
                    });
                }
                else {
                    assertHasFixer(param, name);
                }
                let promise;
                if (inputPath !== undefined) {
                    promise = textlint.fixFile(inputPath);
                }
                else if (text !== undefined) {
                    promise = textlint.fixText(text, ext);
                }
                else {
                    throw new Error("Should set `text` or `inputPath`");
                }
                return promise.then((result) => {
                    const output = invalid.output;
                    assert.strictEqual(result.output, output);
                });
            });
        }
    }
    /**
     * run test for textlint rule.
     * @param {string} name name is name of the test or rule
     * @param {TextlintRuleModule|TestConfig} param param is TextlintRuleCreator or TestConfig
     * @param {string[]|object[]} [valid]
     * @param {object[]} [invalid]
     */
    run(name, param, { valid = [], invalid = [] }) {
        if (isTestConfig(param)) {
            assertTestConfig(param);
            if (valid) {
                valid.forEach((validCase) => {
                    assert.ok(!validCase.hasOwnProperty("options"), "Could not specify options property in valid object when TestConfig was passed. Use TestConfig.rules.options.");
                });
            }
            if (invalid) {
                invalid.forEach((invalidCase) => {
                    assert.ok(!invalidCase.hasOwnProperty("options"), "Could not specify options property in invalid object when TestConfig was passed. Use TestConfig.rules.options.");
                });
            }
        }
        describe(name, () => {
            invalid.forEach((state) => {
                this.testInvalidPattern(name, param, state);
            });
            valid.forEach((state) => {
                this.testValidPattern(name, param, state);
            });
        });
    }
}
exports.TextLintTester = TextLintTester;
//# sourceMappingURL=textlint-tester.js.map